package java.awt;

import javax.accessibility.AccessibleRole;
import javax.accessibility.AccessibleStateSet;
import javax.accessibility.AccessibleValue;
import javax.accessibility.AccessibleContext;
import java.io.ObjectInputStream;
import java.io.IOException;
import java.io.ObjectOutputStream;
import java.awt.event.AdjustmentEvent;
import java.util.EventListener;
import java.awt.peer.ScrollbarPeer;
import javax.accessibility.AccessibleState;
import java.awt.event.AdjustmentListener;
import javax.accessibility.Accessible;

public class Scrollbar extends Component implements Adjustable, Accessible
{
    public static final int HORIZONTAL = 0;
    public static final int VERTICAL = 1;
    int value;
    int maximum;
    int minimum;
    int visibleAmount;
    int orientation;
    int lineIncrement;
    int pageIncrement;
    transient boolean isAdjusting;
    transient AdjustmentListener adjustmentListener;
    private static final String base = "scrollbar";
    private static int nameCounter;
    private static final long serialVersionUID = 8451667562882310543L;
    private int scrollbarSerializedDataVersion;
    
    private static native void initIDs();
    
    public Scrollbar() throws HeadlessException {
        this(1, 0, 10, 0, 100);
    }
    
    public Scrollbar(int n) throws HeadlessException {
        this(n, 0, 10, 0, 100);
    }
    
    public Scrollbar(int orientation, int n, int n2, int n3, int n4) throws HeadlessException {
        this.lineIncrement = 1;
        this.pageIncrement = 10;
        this.scrollbarSerializedDataVersion = 1;
        GraphicsEnvironment.checkHeadless();
        switch (orientation) {
            case 0:
            case 1: {
                this.orientation = orientation;
                this.setValues(n, n2, n3, n4);
                return;
            }
            default: {
                throw new IllegalArgumentException("illegal scrollbar orientation");
            }
        }
    }
    
    @Override
    String constructComponentName() {
        synchronized (Scrollbar.class) {
            return "scrollbar" + Scrollbar.nameCounter++;
        }
    }
    
    @Override
    public void addNotify() {
        synchronized (this.getTreeLock()) {
            if (this.peer == null) {
                this.peer = this.getToolkit().createScrollbar(this);
            }
            super.addNotify();
        }
    }
    
    @Override
    public int getOrientation() {
        return this.orientation;
    }
    
    public void setOrientation(int orientation) {
        synchronized (this.getTreeLock()) {
            if (orientation == this.orientation) {
                return;
            }
            switch (orientation) {
                case 0:
                case 1: {
                    this.orientation = orientation;
                    if (this.peer != null) {
                        this.removeNotify();
                        this.addNotify();
                        this.invalidate();
                    }
                    break;
                }
                default: {
                    throw new IllegalArgumentException("illegal scrollbar orientation");
                }
            }
        }
        if (this.accessibleContext != null) {
            this.accessibleContext.firePropertyChange("AccessibleState", (orientation == 1) ? AccessibleState.HORIZONTAL : AccessibleState.VERTICAL, (orientation == 1) ? AccessibleState.VERTICAL : AccessibleState.HORIZONTAL);
        }
    }
    
    @Override
    public int getValue() {
        return this.value;
    }
    
    @Override
    public void setValue(int n) {
        this.setValues(n, this.visibleAmount, this.minimum, this.maximum);
    }
    
    @Override
    public int getMinimum() {
        return this.minimum;
    }
    
    @Override
    public void setMinimum(int n) {
        this.setValues(this.value, this.visibleAmount, n, this.maximum);
    }
    
    @Override
    public int getMaximum() {
        return this.maximum;
    }
    
    @Override
    public void setMaximum(int n) {
        if (n == Integer.MIN_VALUE) {
            n = -2147483647;
        }
        if (this.minimum >= n) {
            this.minimum = n - 1;
        }
        this.setValues(this.value, this.visibleAmount, this.minimum, n);
    }
    
    @Override
    public int getVisibleAmount() {
        return this.getVisible();
    }
    
    @Deprecated
    public int getVisible() {
        return this.visibleAmount;
    }
    
    @Override
    public void setVisibleAmount(int n) {
        this.setValues(this.value, n, this.minimum, this.maximum);
    }
    
    @Override
    public void setUnitIncrement(int lineIncrement) {
        this.setLineIncrement(lineIncrement);
    }
    
    @Deprecated
    public synchronized void setLineIncrement(int lineIncrement) {
        lineIncrement = ((lineIncrement < 1) ? 1 : lineIncrement);
        if (this.lineIncrement == lineIncrement) {
            return;
        }
        this.lineIncrement = lineIncrement;
        ScrollbarPeer scrollbarPeer = (ScrollbarPeer)this.peer;
        if (scrollbarPeer != null) {
            scrollbarPeer.setLineIncrement(this.lineIncrement);
        }
        return;
    }
    
    @Override
    public int getUnitIncrement() {
        return this.getLineIncrement();
    }
    
    @Deprecated
    public int getLineIncrement() {
        return this.lineIncrement;
    }
    
    @Override
    public void setBlockIncrement(int pageIncrement) {
        this.setPageIncrement(pageIncrement);
    }
    
    @Deprecated
    public synchronized void setPageIncrement(int pageIncrement) {
        pageIncrement = ((pageIncrement < 1) ? 1 : pageIncrement);
        if (this.pageIncrement == pageIncrement) {
            return;
        }
        this.pageIncrement = pageIncrement;
        ScrollbarPeer scrollbarPeer = (ScrollbarPeer)this.peer;
        if (scrollbarPeer != null) {
            scrollbarPeer.setPageIncrement(this.pageIncrement);
        }
        return;
    }
    
    @Override
    public int getBlockIncrement() {
        return this.getPageIncrement();
    }
    
    @Deprecated
    public int getPageIncrement() {
        return this.pageIncrement;
    }
    
    public void setValues(int n, int visibleAmount, int minimum, int maximum) {
        int value;
        synchronized (this) {
            if (minimum == Integer.MAX_VALUE) {
                minimum = 2147483646;
            }
            if (maximum <= minimum) {
                maximum = minimum + 1;
            }
            long n2 = (long)maximum - (long)minimum;
            if (n2 > 2147483647L) {
                n2 = 2147483647L;
                maximum = minimum + (int)n2;
            }
            if (visibleAmount > (int)n2) {
                visibleAmount = (int)n2;
            }
            if (visibleAmount < 1) {
                visibleAmount = 1;
            }
            if (n < minimum) {
                n = minimum;
            }
            if (n > maximum - visibleAmount) {
                n = maximum - visibleAmount;
            }
            value = this.value;
            this.value = n;
            this.visibleAmount = visibleAmount;
            this.minimum = minimum;
            this.maximum = maximum;
            ScrollbarPeer scrollbarPeer = (ScrollbarPeer)this.peer;
            if (scrollbarPeer != null) {
                scrollbarPeer.setValues(n, this.visibleAmount, minimum, maximum);
            }
        }
        if (value != n && this.accessibleContext != null) {
            this.accessibleContext.firePropertyChange("AccessibleValue", Integer.valueOf(value), Integer.valueOf(n));
        }
    }
    
    public boolean getValueIsAdjusting() {
        return this.isAdjusting;
    }
    
    public void setValueIsAdjusting(boolean isAdjusting) {
        int isAdjusting2;
        synchronized (this) {
            isAdjusting2 = this.isAdjusting;
            this.isAdjusting = isAdjusting;
        }
        if (isAdjusting2 != (isAdjusting ? 1 : 0) && this.accessibleContext != null) {
            this.accessibleContext.firePropertyChange("AccessibleState", isAdjusting2 ? AccessibleState.BUSY : null, isAdjusting ? AccessibleState.BUSY : null);
        }
    }
    
    @Override
    public synchronized void addAdjustmentListener(AdjustmentListener adjustmentListener) {
        if (adjustmentListener == null) {
            return;
        }
        this.adjustmentListener = AWTEventMulticaster.add(this.adjustmentListener, adjustmentListener);
        this.newEventsOnly = true;
        return;
    }
    
    @Override
    public synchronized void removeAdjustmentListener(AdjustmentListener adjustmentListener) {
        if (adjustmentListener == null) {
            return;
        }
        this.adjustmentListener = AWTEventMulticaster.remove(this.adjustmentListener, adjustmentListener);
        return;
    }
    
    public synchronized AdjustmentListener[] getAdjustmentListeners() {
        return (AdjustmentListener[])this.getListeners(AdjustmentListener.class);
    }
    
    @Override
    public <T extends EventListener> T[] getListeners(Class<T> clazz) {
        if (clazz == AdjustmentListener.class) {
            return AWTEventMulticaster.getListeners(this.adjustmentListener, clazz);
        }
        return super.getListeners(clazz);
    }
    
    @Override
    boolean eventEnabled(AWTEvent awtEvent) {
        if (awtEvent.id == 601) {
            return (this.eventMask & 0x100L) != 0x0L || this.adjustmentListener != null;
        }
        return super.eventEnabled(awtEvent);
    }
    
    @Override
    protected void processEvent(AWTEvent awtEvent) {
        if ((awtEvent instanceof AdjustmentEvent)) {
            this.processAdjustmentEvent((AdjustmentEvent)awtEvent);
            return;
        }
        super.processEvent(awtEvent);
        return;
    }
    
    protected void processAdjustmentEvent(AdjustmentEvent adjustmentEvent) {
        AdjustmentListener adjustmentListener = this.adjustmentListener;
        if (adjustmentListener != null) {
            adjustmentListener.adjustmentValueChanged(adjustmentEvent);
        }
    }
    
    @Override
    protected String paramString() {
        return super.paramString() + ",val=" + this.value + ",vis=" + this.visibleAmount + ",min=" + this.minimum + ",max=" + this.maximum + ((this.orientation == 1) ? ",vert" : ",horz") + ",isAdjusting=" + this.isAdjusting;
    }
    
    private void writeObject(ObjectOutputStream objectOutputStream) throws IOException {
        objectOutputStream.defaultWriteObject();
        AWTEventMulticaster.save(objectOutputStream, "adjustmentL", this.adjustmentListener);
        objectOutputStream.writeObject(null);
    }
    
    private void readObject(ObjectInputStream objectInputStream) throws ClassNotFoundException, IOException, HeadlessException {
        GraphicsEnvironment.checkHeadless();
        objectInputStream.defaultReadObject();
        Object object;
        while (null != (object = objectInputStream.readObject())) {
            if ("adjustmentL" == ((String)object).intern()) {
                this.addAdjustmentListener((AdjustmentListener)(AdjustmentListener)objectInputStream.readObject());
            }
            else {
                objectInputStream.readObject();
            }
        }
    }
    
    @Override
    public AccessibleContext getAccessibleContext() {
        if (this.accessibleContext == null) {
            this.accessibleContext = new Scrollbar.AccessibleAWTScrollBar();
        }
        return this.accessibleContext;
    }
    
    static {
        Scrollbar.nameCounter = 0;
        Toolkit.loadLibraries();
        if (!GraphicsEnvironment.isHeadless()) {
            initIDs();
        }
    }
    
    protected class AccessibleAWTScrollBar extends Component.AccessibleAWTComponent implements AccessibleValue
    {
        private static final long serialVersionUID = -344337268523697807L;
        
        @Override
        public AccessibleStateSet getAccessibleStateSet() {
            AccessibleStateSet accessibleStateSet = super.getAccessibleStateSet();
            if (Scrollbar.this.getValueIsAdjusting()) {
                accessibleStateSet.add(AccessibleState.BUSY);
            }
            if (Scrollbar.this.getOrientation() == 1) {
                accessibleStateSet.add(AccessibleState.VERTICAL);
                return accessibleStateSet;
            }
            accessibleStateSet.add(AccessibleState.HORIZONTAL);
            return accessibleStateSet;
        }
        
        @Override
        public AccessibleRole getAccessibleRole() {
            return AccessibleRole.SCROLL_BAR;
        }
        
        @Override
        public AccessibleValue getAccessibleValue() {
            return this;
        }
        
        @Override
        public Number getCurrentAccessibleValue() {
            return Integer.valueOf(Scrollbar.this.getValue());
        }
        
        @Override
        public boolean setCurrentAccessibleValue(Number n) {
            if ((n instanceof Integer)) {
                Scrollbar.this.setValue(n.intValue());
                return true;
            }
            return false;
        }
        
        @Override
        public Number getMinimumAccessibleValue() {
            return Integer.valueOf(Scrollbar.this.getMinimum());
        }
        
        @Override
        public Number getMaximumAccessibleValue() {
            return Integer.valueOf(Scrollbar.this.getMaximum());
        }
    }
}
